#include <u.h>
#include <libc.h>
#include <ctype.h>
#include "misc.h"


int
xstrcmp(char *s, char *t)
{
	if(s == nil){
		if(t)
			return -1;
		return 0;
	}
	if(t)
		return strcmp(s,t);
	return 1;
}

char*
trim(char *s)
{
	char *t;
	while(isspace(*s))
		s++;
	t = strchr(s, 0);
	t--;
	while(isspace(*t ))
		t--;
	t++;
	*t = 0;
	return s;
}

#ifdef nodef
*	shell style path matching function
*	I modified to simplify original one
*	and made some modification of the matching rule.
*	now, '/' is a simplly regular charactor except
*	"/*/" is match to "/".
*	ref: /sys/src/cmd/rc/glob.c
*	-Kenar-
*
 * 	Does the string s match the pattern p
 * 	* matches any sequence of characters
 * 	? matches any single character
 * 	[...] matches the enclosed list of characters
 *	~ in the beginning of [...] means compliment
 *	- in [...] means range in ascii order
 *
 *	return value
 *	1: matched
 *	0: not matched
 #endif
int
match(char *s, char *p)
{
	char *s0;
	int compl, hit, lo, hi, t, c;
	s0 = s;
	for(;*p; s++,p++){
		switch(*p){
		case '*':
			++p;
			if(*p == '/' && s > s0)
				if(match(s-1, p)) return 1;
			for(;;){
				if(match(s, p)) return 1;
				if(!*s) break;
				s++;
			}
			return 0;
		case '?':
			if(*s=='\0') return 0;
			break;
		case '[':
			if(*s=='\0') return 0;
			c=*s;
			p++;
			compl=*p=='~';
			if(compl) p++;
			hit=0;
			while(*p!=']'){
				if(*p=='\0') return 0;		/* syntax error */
				lo=*p;
				p++;
				if(*p!='-') hi=lo;
				else{
					p++;
					if(*p=='\0') return 0;	/* syntax error */
					hi=*p;
					p++;
					if(hi<lo){ t=lo; lo=hi; hi=t; }
				}
				if(lo<=c && c<=hi) hit=1;
			}
			if(compl) hit=!hit;
			if(!hit) return 0;
			break;
		default:
			if(*p != *s) return 0;
		}
	}
	return *s=='\0';
}


char *
skip(char *s, char *p)
{
	while(*s && strchr(p, *s))
		s++;
	return s;
}

char *
skipto(char *s, char *p)
{
	while(*s && !strchr(p, *s))
		s++;
	return s;
}

/*	getword get a word from s and returns point
*	next to search ( the point is next char of the word)
*	if there is no word, it returns nil
*	-Kenar-	*/
char *
getword(char *s, char *w, int size)
{
	char *r;
	r = w + size;
	s = skip(s, " \t");
	if(*s == 0)
		return nil;
	while(*s && !strchr(" \t", *s)){
		if(w == r){
			w--;
			break;
		}
		*w++ = *s++;
	}
	*w = 0;
	return s;
}

/*
*	exeecutable returns 1
*	if the file is executable for `others'
*	else returns 0
*/
int
executable(char *path)
{
	Dir *dir;
	dir = dirstat(path);
	if(dir == nil)
		return 0;
	if(dir->mode & DMEXEC)
		return 1;
	free(dir);
	return 0;
}

int
waitfor(int pid, char *msg)
{	int cpid;
	Waitmsg *w;
	
	while((w = wait()) != nil){
		if(pid == w->pid){
			strncpy(msg, w->msg, ERRLEN);
			free(w);
			return 0;
		}
		free(w);
	}
	return -1;
}

