#include <u.h>
#include <libc.h>
#include <auth.h>
#define ERRLEN 256

static char *LOG = "mon";
static char *usage="usage: mon [-d] [-u user] [-p passwdfile] [-r req] cmdpath argument ...";
/*
*	-d: run as daemon mode
*	-u user: run as user
*	-p passwdfile: password file for the user
*	-r req: factotum format of X.509 certificate signing request 
*/

static char *user=nil;
static char *password=nil;
static char *passwdfile=nil;
static int daemon = 0;

static void	execv0(char *argv[]);
static int	waitfor(int pid, char *msg);
static void	becomenone(void);

void
setfactotum(char *req)
{
	int fd;
	int pid, status;
	int n;
	char msg[ERRLEN];
	char *fact;
	switch(pid = fork()) {/* assign = */
	case -1:
		sysfatal("fork");
	case 0:
		close(0);
		close(1);
		close(2);
		execl("/bin/auth/factotum", "factotum", "-n", nil);
		syslog(0, LOG, "fail in execution of factotum");
		sysfatal("fail in execution of factotum");
	default:
		break;
	}
	/* ensure "/bin/auth/factotum" finished */
	status = waitfor(pid, msg);
	if(status < 0){
		syslog(0,LOG,"error: setfactotum: no children to wait %d", pid);
		sysfatal("error: setfactotum: waitfor: %r");
	}
	fact = "/mnt/factotum/ctl";
	fd = open(fact, OWRITE);
	if(fd < 0){
		syslog(0, LOG, "unable to open %s: %r", fact);
		sysfatal("unable to open %s: %r", fact);
	}
	seek(fd, 0, 2);
	n = write(fd, req, strlen(req));
	if(n != strlen(req)){
		syslog(0, LOG, "unable to write %s: %r", fact);
		sysfatal("unable to write %s: %r", fact);
	}
	close(fd);
}

void
main(int argc, char **argv)
{
	int n, t0, t, pid;
	char buf[1024*4], msg[ERRMAX], *p;
	char *user0; // who invoked
	char *req = nil; // request for certificate
	int fd;

	ARGBEGIN{
	case 'u':
		user = ARGF();
		break;
	case 'p':
		passwdfile = ARGF();
		break;
	case 'd':
		daemon = 1;
		break;
	case 'r':
		req = ARGF();
		break;
	default:
		sysfatal("%s", usage);
	}ARGEND

	if(*argv == nil)
		sysfatal("%s", usage);
	user0 = getuser();
	if(user && strcmp(user,user0) != 0){
		char *p;
		if(!passwdfile)
			sysfatal("no passwd file given");
		fd = open(passwdfile, OREAD);
		if(fd < 0)
			sysfatal("not found %s: %r", passwdfile);
		n = read(fd, buf, sizeof buf);
		buf[n] = 0;
		p = strchr(buf, '\n');
		if(p)
			*p = 0;
		close(fd);
		password = strdup(buf);
	}

	if(req){
		fd = open(req, OREAD);
		if(fd < 0)
			sysfatal("not found %s: %r", req);
		n = read(fd, buf, sizeof buf);
		if( n == sizeof buf)
			sysfatal("size of %s over %d: %r", req, sizeof buf);
		buf[n] = 0;
		req = strdup(buf);
		close(fd);
	}

	if(daemon){
		switch(rfork(RFNOTEG|RFPROC|RFFDG|RFNAMEG)) {
		case -1:
			sysfatal("fork");
		case 0:
			break;
		default:
			exits(nil);
		}
	}
	/*
	 * open all files we might need before castrating namespace
	 */
	time(nil);
	syslog(0, LOG, nil);
	syslog(0, LOG, "mon start %s by %s run as %s", *argv, user0, user);

	n = 0;
	t0 = time(nil);
	while(n < 3){
		/*	child should be in copied name space	*/
		switch(pid = rfork(RFFDG|RFNOTEG|RFREND|RFPROC|RFNAMEG)){// assign =
		case -1:
			sysfatal("fork: %r");
		case 0:
			if(user){
				if(strcmp(user, user0) != 0
				&& login(user, password, nil)){
					syslog(0,LOG,"login failure");
					sysfatal("login failure %s %s", user, password);
				}
			}
			else
				becomenone();
			if(req)
				setfactotum(req);
			execv0(argv);
			exits(nil);
		default:
			syslog(0, LOG, "waiting %d",pid);
			waitfor(pid, msg);
			if(*msg){
				p = strchr(msg,':'); p = strchr(p, ':'); p++;
				syslog(0,LOG,"%d:%s", pid, p);
			}
			else
				syslog(0,LOG,"terminated");
		}
		t = time(nil);
		if(t -t0 < 5) // something wrong
			n++;
		else
			n = 0;
		t0 = t;
	}
}

static void
becomenone(void)
{
	int fd;

	fd = open("#c/user", OWRITE);
	if(fd < 0 || write(fd, "none", strlen("none")) < 0)
		sysfatal("can't become none");
	close(fd);
	if(newns("none", nil) < 0)
		sysfatal("can't build normal namespace");
}


static void
execv0(char *argv[])
{
	char path[256], wbuf[256],*p;
	if(*argv[0] == '/')
		strcpy(path, argv[0]);
	else if(strncmp(argv[0], "./",2) == 0){
		argv[0]++;
		snprint(path, sizeof path, "%s%s", getwd(wbuf, sizeof wbuf),argv[0]);
	}
	else
		snprint(path, sizeof path, "/bin/%s", argv[0]);
	p = strrchr(path, '/');
	if(p){
		p++;
		argv[0] = p;
	}
	exec(path, argv);
	sysfatal("unable to exec: %s: %r", path);
}

static int
waitfor(int pid, char *msg)
{
	Waitmsg *w;
	while((w = wait()) != nil){
		if(w->pid == pid){
			strncpy(msg, w->msg, ERRMAX);
			free(w);
			return 0;
		}
		free(w);
	}
	return -1;
}
